package com.bleujin.framework.configuration;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Configuration Factory Ŭ<br>
 * 
 * <pre>
 * Configuration instance . factory νϽ static  ׻ ϳ instance 
 * Ѵ.
 * 
 * Instance 
 * ex&gt;
 *      ConfigurationFactory factory = ConfigurationFactory.newinstance();
 * 
 * Configuration ϱ
 * ex&gt;
 *      factory.getConfiguration( &quot;elementPathString&quot; ); // elementPathString &lt;config&gt;&lt;sys&gt;... ϰ &quot;config.sys. &quot; Ͱ ǥѴ.
 * 
 * 
 * 
 * </pre>
 * 
 * @author Choi sei hwan <a href="mailto:sehan@i-on.net">Choi sei hwan</a>
 * @version 1.0
 */

public class DefaultConfigurationFactory extends ConfigurationFactory {
	private Element documentElement;

	//  Configuration νϽ ϴ hashtable
	private Hashtable configurationCache = new Hashtable();

	private ConfigurationType configurationType = ConfigurationType.DEFAULT;

	/**
	 * instanceKeyName   νϽ Ѵ.
	 * 
	 * @param instanceKeyName
	 *            String
	 */
	public DefaultConfigurationFactory(String instanceKeyName) {
		if (instanceKeyName == null) {
			this.instanceKeyName = "default_instanceKeyName";
		} else {
			this.instanceKeyName = instanceKeyName;
		}

		// System.out.println( "Configuration Factory instance Create! ( key : + instanceKeyName + " )" );
	}

	/**
	 * DocumentBuilder  Ѵ.
	 * 
	 * @throws ConfigurationBuildException
	 * @return DocumentBuilder
	 */
	private DocumentBuilder createDocumentBuilder() throws ConfigurationBuildException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = null;

		try {
			builder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException ex) {
			throw new ConfigurationBuildException(ex.getLocalizedMessage(), ex);
		}
		return builder;

	}

	// /**
	// * Ϸ  DocumentElement  ´.
	// *
	// * @param configFile - ȯ漳  XML νϽ
	// * @throws ConfigurationBuildException
	// */
	// private void createDocumentFromFile( File configFile ) throws
	// ConfigurationBuildException
	// {
	// Document doc = null;
	//
	// try
	// {
	// doc = createDocumentBuilder().parse( configFile );
	// }
	// catch( IOException ex1 )
	// {
	// throw new ConfigurationBuildException( ex1.getLocalizedMessage(), ex1 );
	// }
	// catch( SAXException ex1 )
	// {
	// throw new ConfigurationBuildException( ex1.getLocalizedMessage(), ex1 );
	// }
	//
	// documentElement = doc.getDocumentElement();
	// }

	/**
	 * org.xml.sax.InputSource   DocumentElement  ´.
	 * 
	 * @param inputsource InputSource ȯ漳  InputSource νϽ
	 * @throws ConfigurationBuildException
	 */
	private void createDocument(InputSource inputsource) throws ConfigurationBuildException {
		Document doc = null;

		try {
			doc = createDocumentBuilder().parse(inputsource);
		} catch (IOException ex1) {
			throw new ConfigurationBuildException(ex1.getLocalizedMessage(), ex1);
		} catch (SAXException ex1) {
			throw new ConfigurationBuildException(ex1.getLocalizedMessage(), ex1);
		}
		documentElement = doc.getDocumentElement();
	}

	// public void build( File configFile ) throws ConfigurationBuildException
	// {
	// createDocumentFromFile( configFile );
	// }

	/**
	 * inputsource  ؼ xml config  оδ.
	 * 
	 * @param inputsource InputSource ȯ漳  InputSource Class
	 * @throws ConfigurationBuildException
	 */
	public void build(InputSource inputsource) throws ConfigurationBuildException {
		createDocument(inputsource);
	}

	/**
	 * ãϴ Element  Root Element  Ȯ
	 * 
	 * @param elementToken ElementInfoTokenizer ãϴ element    ִ ElementInfoTokenizer
	 * @throws NotFoundXmlTagException
	 * @return boolean ãϴ element  root   true ƴϸ false
	 */
	private boolean isRootElement(ElementInfoTokenizer elementToken) throws NotFoundXmlTagException {
		String rootElementName = elementToken.nextToken().getElementName();

		if (!documentElement.getTagName().equalsIgnoreCase(rootElementName)) {
			throw new NotFoundXmlTagException(elementToken.getElementPathString() + ":" + rootElementName);
		}

		if (!elementToken.hasMoreTokens()) {
			// root element
			return true;
		} else {
			// sub element
			return false;
		}
	}

	/**
	 * ó Configuration νϽ Hashtable ߰Ѵ.ѹ Configuration νϽ
	 * Hashtable ߰ȴ.
	 * 
	 * @param key
	 *            String Configuration νϽ Ű. elementPathString εȴ.
	 * @param obj
	 *            Object  Configuration νϽ
	 */
	private void addConfigurationCache(String key, Object obj) {
		configurationCache.put(key, obj);
	}

	/**
	 * Hashtable ˻ؼ Configuration returnѴ.
	 * 
	 * @param key
	 *            String Configuration νϽ Ű. elementPathString εȴ.
	 * @return Object Object( Configuration νϽ )
	 */
	private Object searchConfigurationCache(String key) {
		return configurationCache.get(key);
	}

	/**
	 * ConfigurationFactory build ޼ҵ尡 Ǿ üũѴ. build޼ҵ尡 Ǿ
	 * documentElement  Ǿ θ üũ.
	 * 
	 * @return boolean build Ǿ true, ׷  false
	 */
	private boolean isBuild() {
		if (this.documentElement != null)
			return true;
		else
			return false;
	}

	/**
	 * elementPathString شϴ Configuration νϽ ´.
	 * 
	 * @param elementPathString
	 *            String  Configuration νϽ path ڿ
	 * @throws NotFoundXmlTagException
	 * @throws NotBuildException
	 * @throws ConfigurationException
	 * @return Configuration
	 */
	public Configuration getConfiguration(String elementPathString) throws NotFoundXmlTagException, NotBuildException, ConfigurationException {
		return getConfiguration(elementPathString, null);
	}

	/**
	 * θ𿡼 ڽ Element  Configuration  ϳ ´. ϳ̻ ϸ   ߰ߵǴ° ´.
	 * 
	 * @param childElementName
	 *            String ڽ element  ̸
	 * @param parentConfiguration
	 *            DefaultConfiguration θ Configuration νϽ
	 * @throws NotFoundXmlTagException
	 * @throws NotBuildException
	 * @throws ConfigurationException
	 * @return Configuration ڽ Element  Configuration νϽ
	 */
	public Configuration getConfiguration(String childElementName, DefaultConfiguration parentConfiguration) throws NotFoundXmlTagException, NotBuildException, ConfigurationException {
		return (this.getConfigurations(childElementName, parentConfiguration))[0];

	}

	/**
	 * elementPathString شϴ Configuration νϽ ´.
	 * 
	 * @param elementPathString
	 *            String  Configuration νϽ path ڿ
	 * @throws NotFoundXmlTagException
	 * @throws NotBuildException
	 * @throws ConfigurationException
	 * @return Configuration[]
	 */
	public Configuration[] getConfigurations(String elementPathString) throws NotFoundXmlTagException, NotBuildException, ConfigurationException {
		return getConfigurations(elementPathString, null);
	}

	/**
	 * θ𿡼 ڽ Element  Configuration ´.
	 * 
	 * @param childElementName
	 *            String ڽ element  ̸
	 * @param parentConfiguration
	 *            DefaultConfiguration  Configuration νϽ
	 * @throws NotFoundXmlTagException
	 * @throws NotBuildException
	 * @throws ConfigurationException
	 * @return Configuration[] ڽ Element  Configuration []
	 */
	public Configuration[] getConfigurations(String childElementName, DefaultConfiguration parentConfiguration) throws NotFoundXmlTagException, NotBuildException, ConfigurationException {
		Element currentElement = null;
		String elementIndexStringTemp = null;
		String elementIndexString = null;
		String elementPathString = null;
		int elementDepth = 0;

		// θ Element  Ѵ.
		if (parentConfiguration != null) {
			// θ  parentConfiguration  Element  ã´.
			currentElement = parentConfiguration.getElement();
			elementPathString = parentConfiguration.getElementPathString() + "." + childElementName;
			elementIndexString = parentConfiguration.getElementIndexString();
			elementIndexStringTemp = elementIndexString;
			elementDepth = parentConfiguration.getElementDepth();
		} else {
			//   DocumentElement  ã´.
			currentElement = documentElement;
			elementPathString = childElementName;
			int count = (new StringTokenizer(elementPathString, ".")).countTokens() - 2;
			elementIndexString = "0";
			while (count > 0) {
				elementIndexString += ".0";
				count--;
			}
			// elementIndexString  "" ̸ ֻ element  ϳ̹Ƿ "0"  ⺻ Ѵ.
			elementIndexStringTemp = "";
		}
		// DEBUG
		// System.out.println( "[[ getConfigurations() DEBUG Message Start ]]");
		// System.out.println( "- currentElement : " +
		// currentElement.getTagName());
		// System.out.println( "- elementPathString : " + elementPathString );
		// System.out.println( "- elementIndexString : " + elementIndexString );

		// build Ǿ ˻Ѵ.
		if (!isBuild()) throw new NotBuildException("ConfigurationFactory.build() not execute!!");

		ElementInfoTokenizer elementToken = new ElementInfoTokenizer(elementPathString, elementIndexString);

		// Configuration Cache Table  ˻Ѵ.
		// ã return ׷ġ  configurations ʱȭ
		Vector configurations = null;
		if ((configurations = (Vector) searchConfigurationCache(elementPathString + ":" + elementIndexString)) != null)
			return (Configuration[]) configurations.toArray(new Configuration[0]);
		else
			configurations = new Vector();

		Vector elements = new Vector();

		// root element ̸  element  ã´. elememntToken  next 1ȸ
		if (!isRootElement(elementToken)) {
			// ãϴ Element  Root Element  ƴҶ

			NodeList currentNodeList = null;
			boolean isFindElement = false;
			ElementInfo currentToken = null;
			int indexCount = 0;

			for (int i = 0; i < elementDepth - 1; i++)
				elementToken.nextToken();

			while (elementToken.hasMoreTokens()) {
				isFindElement = false;

				currentToken = elementToken.nextToken();
				// DEBUG
				// System.out.println( "currentToken elementName : " +
				// currentToken.getElementName() );
				// System.out.println( "currentToken elementIndex : " +
				// currentToken.getElementIndex() );

				// xml element  ܿ  ش element  ã´.
				currentNodeList = currentElement.getChildNodes();

				indexCount = 0;
				for (int i = 0; i < currentNodeList.getLength(); i++) {
					// DEBUG
					// System.out.println( "element type : " +
					// currentNodeList.item(i).getNodeType() );
					// System.out.println( "element node name : " +
					// currentNodeList.item(i).getNodeName() );

					if (currentNodeList.item(i).getNodeType() == NODETYPE.ELEMENT && currentNodeList.item(i).getNodeName().equalsIgnoreCase(currentToken.getElementName())) {
						if (elementToken.hasMoreTokens()) {
							// DEBUG
							// System.out.println( "element name : " +
							// currentNodeList.item( i ).getNodeName() );
							if (indexCount == currentToken.getElementIndex()) {
								currentElement = (Element) currentNodeList.item(i);
								i = currentNodeList.getLength();

								// elementIndexString 
								if (elementIndexStringTemp != "")
									elementIndexStringTemp += ".";
								elementIndexStringTemp += currentToken.getElementIndex();

								isFindElement = true;
							}

							indexCount++;
						} else {
							// DEBUG
							// System.out.println( "elements add : " + ( (
							// Element )currentNodeList.item( i ) ).getTagName()
							// );
							elements.add((Element) currentNodeList.item(i));
							isFindElement = true;
						}

					}
				}

				if (!isFindElement) {
					throw new NotFoundXmlTagException(elementPathString + ":" + currentToken.getElementName() + "( " + (currentToken.getElementIndex() + 1) + " )");
				}
			}
		} else {
			// ãϴ Element  Root Element ϶
			elements.add(documentElement);
		}

		// DEBUG
		// System.out.println( "elementIndexStringTemp : " + elementIndexStringTemp );

		// Element obj  ִ elements Vector Configuration  obj  Vector Ѵ.
		setElementToConfiguration(elementPathString, elementIndexStringTemp, elements, configurations);

		// Configuration Cache Table  Configurations Object  Ѵ.
		addConfigurationCache(elementPathString + ":" + elementIndexStringTemp, configurations);

		return (Configuration[]) configurations.toArray(new Configuration[0]);
	}

	/**
	 * elements Vector  Element type Object  Configuration type  Object 
	 * ȯŲ.
	 * 
	 * @param elementPathString
	 *            String  Configuration νϽ path ڿ
	 * @param elementIndexString
	 *            String ܰ IndexString
	 * @param elements
	 *            Vector Element   Vector
	 * @param configurations
	 *            Vector Configuration   Vector
	 * @throws ConfigurationException
	 */
	private void setElementToConfiguration(final String elementPathString, final String elementIndexString, Vector elements, Vector configurations) throws ConfigurationException {
		Class configurationClass = null;
		Constructor configurationConstructor = null;
		//  Ķ  Class
		Class[] parameterTypes = new Class[] { Element.class, String.class, String.class, String.class };
		Object[] parameterValues = null;
		// elementPathString  ´ Configuration Instance  Configurations ִ´.
		for (int j = 0; j < elements.size(); j++) {
			try {
				configurationClass = Class.forName(configurationType.getClassName());
			} catch (ClassNotFoundException ex) {
				throw new ConfigurationException("Class not found : " + configurationType.getClassName(), ex);
			}

			try {
				configurationConstructor = configurationClass.getConstructor(parameterTypes);
			} catch (SecurityException ex1) {
				throw new ConfigurationException(ex1.getLocalizedMessage(), ex1);
			} catch (NoSuchMethodException ex1) {
				throw new ConfigurationException("No such method : " + configurationType.getClassName() + " Constructor", ex1);
			}

			//  Ķ value Object
			String elementIndexStringTemp = null;
			if (elementIndexString == "")
				elementIndexStringTemp = String.valueOf(j);
			else
				elementIndexStringTemp = elementIndexString + "." + String.valueOf(j);

			parameterValues = new Object[] { (Element) elements.get(j), elementPathString, elementIndexStringTemp, this.instanceKeyName };
			try {
				configurations.add(configurationConstructor.newInstance(parameterValues));
			} catch (Exception ex) {
				throw new ConfigurationException(ex.getLocalizedMessage(), ex);
			}

		}
	}

	public Hashtable getConfigurationCache() {
		return this.configurationCache;
	}

}

/**
 * <pre>
 * (.)  еǾ ڿ elementPath  elementIndex ιڿ ޾
 * ElementInfo · Token  ش.
 * elementPath  elementIndex  ġ ִ token  ϳ ElementInfo  Token ȴ.
 * 
 * ex)
 *      elementPath = &quot;aaaa.bbbb.cccc&quot;;
 *      elementIndex = &quot;0.1.0&quot;;
 *      ̶
 *      {aaaa, 0 }, {bbbb, 1}, {cccc, 0}    ϳ ElementInfo ̷.
 * 
 * 
 * </pre>
 * 
 * @author Choi sei hwan <a href="mailto:sehan@i-on.net">Choi sei hwan</a>
 * @version 1.0
 */
class ElementInfoTokenizer {
	private StringTokenizer pathToken, indexToken;
	private String elementPath, elementIndex;

	public ElementInfoTokenizer(String elementPath, String elementIndex) {
		this.elementPath = elementPath;
		pathToken = new StringTokenizer(this.elementPath, ".");

		this.elementIndex = elementIndex;
		indexToken = new StringTokenizer(this.elementIndex, ".");
	}

	/**
	 * ElementIndexString 
	 * 
	 * @return String
	 */
	public String getElementIndexString() {
		return this.elementIndex;
	}

	/**
	 * ElementPathString 
	 * 
	 * @return String
	 */
	public String getElementPathString() {
		return this.elementPath;
	}

	/**
	 * ElementInfoToken   ´.
	 * 
	 * @return ElementInfoToken 
	 */
	public int countTokens() {
		return pathToken.countTokens();
	}

	/**
	 *  ElementInfoToken  ´.
	 * 
	 * @return ElementInfo  ElementInfoToken
	 */
	public ElementInfo nextToken() {
		int index = 0;
		if (indexToken.hasMoreTokens()) {
			index = Integer.parseInt(indexToken.nextToken());
		}
		return new ElementInfo(pathToken.nextToken(), index);
	}

	/**
	 * Token  ִ üũѴ.
	 * 
	 * @return boolean  true ׷  false
	 */
	public boolean hasMoreTokens() {
		return pathToken.hasMoreTokens();
	}

}

/**
 * elementName  elementIndex ѽ  ȴ. elementName, elementIndex  ü
 * 
 * @author Choi sei hwan <a href="mailto:sehan@i-on.net">Choi sei hwan</a>
 * @version 1.0
 */
class ElementInfo {
	private String elementName;
	private int elementIndex;

	public ElementInfo(String curElementName, int curElementIndex) {
		this.elementName = curElementName;
		this.elementIndex = curElementIndex;
	}

	/**
	 * ElementName ´.
	 * 
	 * @return String
	 */
	public String getElementName() {
		return this.elementName;
	}

	/**
	 * ElementIndex ´.
	 * 
	 * @return int
	 */
	public int getElementIndex() {
		return this.elementIndex;
	}

}
