package com.bleujin.framework.db;

import java.sql.SQLException;

import com.bleujin.framework.configuration.Configuration;
import com.bleujin.framework.configuration.ConfigurationException;
import com.bleujin.framework.configuration.NotFoundXmlTagException;
import com.bleujin.framework.db.constant.ILogger;
import com.bleujin.framework.db.constant.RuntimeService;
import com.bleujin.framework.db.manager.DBManager;
import com.bleujin.framework.db.manager.OracleDBManager;
import com.bleujin.framework.db.procedure.CombinedUserProcedures;
import com.bleujin.framework.db.procedure.IParameterQueryable;
import com.bleujin.framework.db.procedure.IQueryable;
import com.bleujin.framework.db.procedure.IUserCommand;
import com.bleujin.framework.db.procedure.IUserCommandBatch;
import com.bleujin.framework.db.procedure.IUserProcedure;
import com.bleujin.framework.db.procedure.IUserProcedureBatch;
import com.bleujin.framework.db.procedure.RepositoryService;
import com.bleujin.framework.db.procedure.UserProcedures;
import com.bleujin.framework.db.servant.AfterTask;
import com.bleujin.framework.db.servant.ExtraServant;
import com.bleujin.framework.db.servant.NoneServant;
import com.bleujin.framework.db.servant.PrimaryServant;
import com.bleujin.framework.db.servant.ServantChannel;
import com.bleujin.framework.util.InstanceCreationException;
import com.bleujin.framework.util.InstanceCreator;

/**
 * Front Facade Role
 * Used Facade Pattern
 * .execUpdate, .execQuery(String DynamicQuery) 
 * .execCommandUpdate, .execCommandQuery(Parameter Query)
 * .execProcedureUpdate, .execProcedureQuery(Procedure Query & Lob Type Query)
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: I-ON Communications</p>
 * @author bleujin
 * @version 1.0
 */

public class DBController implements IDBController{ // implements Configurable
//    public static DBController TEST_CONTROLLER = new DBController("test", new Oracle9icDBManager("jdbc:oracle:thin:@novision:1521:bleujin", "al", "redf"), new StdOutServant(StdOutServant.All)) ;
//    public static DBController TEST_CONTROLLER = new DBController("test", new Oracle9iCacheDBManager("jdbc:oracle:thin:@novision:1521:bleujin", "al", "redf"), new NoneServant()) ;

    protected final String name ;
    protected final DBManager dbm ;
    protected int limitedRows = Page.DEFAULT.getListNum();
    private ServantChannel echannel = null;


    private MidgardProperty midgard = MidgardProperty.EMPTY ;    // etc property...
    private PrimaryServant pservant = PrimaryServant.EMPTY_PRIMARYSERVANT ;

    private ILogger log = null ;
    public DBController(Configuration config) throws DBControllerInstantiationException {

        if (!config.getTagName().equals("database-controller"))
            throw new DBControllerInstantiationException("invalid configuration.");

        try {
            // ݵ ʿ ͵...
            this.name = config.getChild("controller-name").getValue();
            this.dbm = (DBManager) InstanceCreator.createConfiguredInstance(config.getChild("database-manager.configured-object"));


            Configuration[] configOfServant = config.getChildren("extra-servant.configured-object");
            ExtraServant eservant = null;

            // None Channel...
            if (configOfServant == null || configOfServant.length <= 0) {
                echannel = new ServantChannel(new NoneServant());
            } else {
                for (int i = 0; i < configOfServant.length; i++) {
                    if (i == 0) {
                        eservant = (ExtraServant) InstanceCreator.createConfiguredInstance(configOfServant[i]);
                        echannel = new ServantChannel(eservant);
                        eservant.setChannel(echannel);
                    } else {
                        ExtraServant nextServant = (ExtraServant) InstanceCreator.createConfiguredInstance(configOfServant[i]);
                        eservant.setNext(nextServant);
                        eservant = nextServant;
                    }
                }
            }

            //    ׸...
            try {

                // add index query at existing query..
                Configuration configOfPServant = config.getChild("primary-servant.configured-object");
                if (configOfPServant != null){
                    pservant = (PrimaryServant) InstanceCreator.createConfiguredInstance(configOfPServant) ;
                }

            } catch (NotFoundXmlTagException ignore) {
            }
//            try {
//                this.limitedRows = config.getChild( "limited-rows" ).getValueAsInt();
//            } catch( NotFoundXmlTagException ex2 ){
//            }

            // midgard
            try{
                Configuration[] mconfig =  config.getChildren("midgard.property") ;
                midgard = new MidgardProperty(mconfig) ;
            } catch (NotFoundXmlTagException ignore){
            }
        } catch (NotFoundXmlTagException ex) {
            throw new DBControllerInstantiationException("Few parameter to initialize DBController.", ex);
        } catch (ConfigurationException e) {
            throw new DBControllerInstantiationException("unexpected exception.", e);
        } catch (InstanceCreationException ice) {
            throw new DBControllerInstantiationException("could not instantiate database manager. ", ice);
        }
    }

    /**
     * @param name controller's name : recognize dbc, but dont need unique name  
     * @param dbm
     */
    public DBController(DBManager dbm) {
        this("Anonymous controller", dbm);
    }

    public DBController(String name, DBManager dbm) {
        this(name, dbm, new NoneServant());
    }

    public DBController(String name, DBManager dbm, ExtraServant servant) {
        this.name = name;
        this.dbm = dbm;
        this.echannel = new ServantChannel(servant);
        servant.setChannel(this.echannel) ;
    }
    
    private boolean initialized = false ; 
    public void initSelf() throws SQLException{
    	this.initSelf(new RuntimeService()) ;
    }
    
    public synchronized void initSelf(RuntimeService rservice) throws SQLException{
    	if (initialized) return ;
    	
    	log = rservice.getLogger() ;
    	getDBManager().initPool(this) ;
    	if (Thread.State.NEW.equals(echannel.getExtraServant().getState())){
            echannel.startWorker();
    	} else {
    		echannel.restartWorker();
    	}
    	this.initialized = true ;
    }

    public MidgardProperty MidgardProperty(){
        return midgard ;
    }

    public DBManager getDBManager() {
        return this.dbm;
    }

    private ExtraServant getServant(){
        return this.echannel.getExtraServant() ;
    }

    private RepositoryService getService() {
        return this.dbm.getRepositoryService() ;
    }

    public int getLimitedRows() {
        return this.limitedRows;
    }
    
    public void setLimitedRows(int limitedRows){
    	this.limitedRows = Math.abs(limitedRows) ;
    }

    public String getName() {
        return this.name;
    }

    public synchronized void destroySelf() {
    	initialized = false ;
        echannel.stopServant();
		try {
	        if (dbm != null) dbm.destroyPool(this);
		} catch (NullPointerException ignore) {
			// already closed 
			log.warning("Destroy of DBController Failed(NullPointer) : " + ignore.getMessage());
		} catch (SQLException ignore) {
			log.warning("Destroy of DBController Failed : " + ignore.getMessage());
		}
        log.info("DBController : destroyed");
    }


    
    // FOR TEST
    public synchronized void changeServant(ExtraServant servant) {
    	this.echannel.stopServant() ;
    	this.echannel = new ServantChannel(servant);
        servant.setChannel(this.echannel) ;
        this.echannel.startWorker() ;
    }
    
    public void addServant(ExtraServant addServant){
    	addServant.setChannel(this.echannel) ;
    	ExtraServant lastServant = echannel.getExtraServant() ;
    	
    	while(lastServant.getNextServant() != null){
    		lastServant = lastServant.getNextServant() ;
    	}
    	lastServant.setNext(addServant) ;
    }
    
    

    
    
//  Standard Query & Update Utililty
    public int execUpdate(String strSQL) throws SQLException {
        return execUpdate(createParameterQuery(strSQL)) ;
    }
    public int execUpdate(IQueryable query) throws SQLException {
        return query.execUpdate() ;
    }

    public Rows getRows(String strSQL) throws SQLException {
    	return getRows(createParameterQuery(strSQL));
    }
    
    public Rows getRows(IQueryable query) throws SQLException {
    	return query.execQuery();
    }
    public Rows getRows(String strSQL, int listNum, int pageNo) throws SQLException {
    	IQueryable query = createParameterQuery(strSQL) ;
    	query.setPage(Page.create(listNum, pageNo)) ;
    	return getRows(query);
    }
// Utility Method End
    
    

    
    // **** impl
    public IUserProcedure createUserProcedure(String proc) {
        IUserProcedure upt = getService().createUserProcedure(this, proc);
        upt.setPage(Page.create(getLimitedRows(), 1)) ;
        return upt ;
    }

    public IParameterQueryable createParameterQuery (String procSQL){
        IParameterQueryable upt = getService().createParameterQuery(this, procSQL) ;
        upt.setPage(Page.create(getLimitedRows(), 1)) ;
        return upt ;
    }

    public IUserCommand createUserCommand(String procSQL) {
    	IUserCommand upt = getService().createUserCommand(this, procSQL);
        upt.setPage(Page.create(getLimitedRows(), 1)) ;
        return upt ;
    }

    public IUserCommandBatch createUserCommandBatch(String procSQL) {
        return getService().createUserCommandBatch(this, procSQL);
    }

    public IUserProcedureBatch createUserProcedureBatch(String procSQL) {
        return getService().createUserProcedureBatch(this, procSQL);
    }
    
    public UserProcedures createUserProcedures(String name) {
    	return getService().createUserProcedures(this, name);
    }
    
    public CombinedUserProcedures createCombinedUserProcedures(String name) {
    	return new CombinedUserProcedures(this, name) ;
    }
    

    
    public void handleServant(long start, long end, IQueryable query, int execType){
        if (query instanceof UserProcedures) {
            UserProcedures uprocs = (UserProcedures)query ;
            for (int i = 0, last = uprocs.size(); i < last; i++) {
                handleServant(start, end, uprocs.getQuery(i), execType);
            }
        }
        else {
            try {
                echannel.putTask(new AfterTask(start, end, this.getDBManager(), query, execType));
            } catch(InterruptedException ex) {
            	ex.printStackTrace() ;
            }
        }
    }
    
    // **** impl end

    



    public String toString(){
    	return this.name + "[" + this.getClass().getName() + "]"; 
    }
    
    
    /* Use ONLY TEST(ex Unit Test)*/
    @Deprecated
    private static DBController TEST_CONSTROLLER ;
    @Deprecated
    public static DBController getTestInstance(){
    	if (TEST_CONSTROLLER == null) 
    		TEST_CONSTROLLER = new DBController("test", new OracleDBManager("jdbc:oracle:thin:@novision:1521:bleujin", "al", "redf"), new NoneServant()) ;
    	return TEST_CONSTROLLER ;
    }


}
