package com.bleujin.framework.db;

import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.sql.RowSet;

import com.bleujin.framework.util.CaseInsensitiveHashMap;


/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: I-ON Communications</p>
 * @author bleujin
 * @version 1.0
 */

public class ExtendTransformer extends Transformer {

    public ExtendTransformer(Rows rows) {
        super(rows);
    }

    public synchronized Map fetchFirstToMap(){
        try {
            if(!rows.first()) {
                throw RepositoryException.throwIt("No Data Found\n");
            }
            return currentRowToMap(rows);
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }


    public synchronized Row fetchFirstToRow(){
        try {
            if(!rows.first()) {
                throw RepositoryException.throwIt("No Data Found\n");
            }
            return fetchToRow(rows);
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    public static Row fetchFirstToRow(Rows rows) {
        try {
            if(!rows.first()) {
                throw RepositoryException.throwIt("No Data Found\n");
            }
            return new Row(currentRowToMap(rows), getColumnsNames(rows));
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    public synchronized List fetchToList() {
        try {
            rows.beforeFirst();
            List results = new ArrayList();

            while(rows.next()) {
                results.add(this.currentRowToMap(rows));
            }

            return results;
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }


    public synchronized Row[] fetchToRowArray() {
        try {
            rows.beforeFirst();
            List results = new ArrayList();

            while(rows.next()) {
                results.add(this.fetchToRow(rows));
            }

            return(Row[])results.toArray(new Row[0]);
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    // current row -> Map
    public static Map currentRowToMap(RowSet rows) {
        try {
            Map result = new CaseInsensitiveHashMap();
            ResultSetMetaData rsmd = rows.getMetaData();
            int cols = rsmd.getColumnCount();

            for(int i = 1; i <= cols; i++) {
                result.put(rsmd.getColumnName(i), getValue(rows, i, rsmd));
            }

            return result;
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    public static Map[] toMap(RowSet rows) {
        try {
            rows.beforeFirst();
            Vector v = new Vector();
            ResultSetMetaData rsmd = rows.getMetaData();
            int cols = rsmd.getColumnCount();

            while(rows.next()) {
                Map map = new CaseInsensitiveHashMap();
                for(int i = 1; i <= cols; i++) {
                    map.put(rsmd.getColumnName(i), getValue(rows, i, rsmd));
                }
                v.add(map);
            }

            return(Map[])v.toArray(new Map[0]);
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    public static Map[] toNullObjectMap(RowSet rows) {
        try {
            rows.beforeFirst();
            Vector v = new Vector();
            ResultSetMetaData rsmd = rows.getMetaData();
            int cols = rsmd.getColumnCount();

            while(rows.next()) {
                Map map = new CaseInsensitiveHashMap();
                for(int i = 1; i <= cols; i++) {
                    map.put(rsmd.getColumnName(i), getNullObjectValue(rows, i, rsmd));
                }
                v.add(map);
            }

            return(Map[])v.toArray(new Map[0]);
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    private static Object getNullObjectValue(RowSet rows, int i, ResultSetMetaData meta) throws SQLException {
        Object value = getValue(rows, i, meta);
        int type = meta.getColumnType(i);
        switch (type) {
            case Types.CHAR:
            case Types.VARCHAR:
            case Types.LONGVARCHAR:
            case Types.CLOB:
                if(value == null) return "";
        }
        return value;
    }

    // current row -> Row
    public synchronized Row fetchToRow(RowSet rows) {
        try {
            return new Row(currentRowToMap(rows), super.getColumnsNames());
        } catch(SQLException ex) {
            throw RepositoryException.throwIt(ex) ;
        }
    }

    private static Object getValue(RowSet rows, int i, ResultSetMetaData meta) throws SQLException {
        if (meta.getColumnType(i) == Types.CLOB) {
            return RowsUtils.clobToString(rows.getClob(i)) ;
        } else {
            return rows.getObject(i);
        }
    }


}
