package com.bleujin.framework.db;

import java.lang.reflect.Array;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import javax.sql.RowSet;

import com.bleujin.framework.util.CaseInsensitiveHashMap;
import com.bleujin.framework.util.InstanceCreationException;
import com.bleujin.framework.util.InstanceCreator;


/**
 * RowSet, Rows  ϱ  · ٲ۴.
 *
 * <p>Title: ICS EV</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: I-ON Communications</p>
 * @author Kim Sanghoon wizest@i-on.net
 * @version 1.0
 */

public class Transformer
{
    protected final Rows rows;
    private String[] columnsNames = null;

    /**
     * @param rowSet
     * @throws SQLException
     */
    public Transformer(Rows rs) {
        this.rows = rs;
    }

    /**
     * @return columns name  빮!
     * @throws SQLException
     */
    public synchronized String[] getColumnsNames() throws SQLException {
        if(this.columnsNames != null)return this.columnsNames;
        return this.columnsNames = getColumnsNames(this.rows);
    }

    public static String[] getColumnsNames(RowSet rs) throws SQLException {
        ResultSetMetaData meta = rs.getMetaData();
        String[] names = new String[meta.getColumnCount()];

        for(int i = 0; i < names.length; ++i) {
            names[i] = meta.getColumnName(i + 1).toUpperCase();
        }

        return names;
    }

    /**
     * row key:columns name , value:columns data  map ǥϿ ü row 迭 Ѵ.
     *
     * @return
     * @throws SQLException
     */
    public synchronized HashMap[] transformIntoHashMaps() throws SQLException {
        return transformIntoHashMaps(this.rows);
    }

    public static HashMap[] transformIntoHashMaps(RowSet rs) throws SQLException {
        Vector maps = new Vector();
        String[] names = getColumnsNames(rs);
        int columnSize = names.length;

        rs.beforeFirst();
        while(rs.next()) {
            HashMap map = new CaseInsensitiveHashMap();
            ArrayList row = RowsUtils.getCurrentTuple(rs);

            for(int i = 0; i < columnSize; ++i) {
                map.put(names[i], row.get(i));
            }

            maps.add(map);
        }

        rs.beforeFirst();
        return(HashMap[])maps.toArray(new CaseInsensitiveHashMap[0]);
    }

    public synchronized HashMap[] transformerIntoHashMapsWithoutNull() throws SQLException {
        return transformerIntoHashMapsWithoutNull(this.rows);
    }

    public static HashMap[] transformerIntoHashMapsWithoutNull(RowSet rs) throws SQLException {
        Vector maps = new Vector();
        String[] names = getColumnsNames(rs);
        int columnSize = names.length;

        rs.beforeFirst();
        while(rs.next()) {
            HashMap map = new CaseInsensitiveHashMap();

            ArrayList row = RowsUtils.getCurrentTuple(rs);

            for(int i = 0; i < columnSize; ++i) {
                Object o = row.get(i);
                if(o == null)o = ""; //  null  "" ִ´.

                map.put(names[i], o);
            }

            maps.add(map);
        }

        rs.beforeFirst();
        return(HashMap[])maps.toArray(new CaseInsensitiveHashMap[0]);
    }

    /**
     *  Ű row key:columns name , value:columns data  map Ѵ.
     *
     * @return
     * @throws SQLException
     */
    public synchronized HashMap transformIntoHashMap() throws SQLException {
        HashMap map = new CaseInsensitiveHashMap();
        String[] names = getColumnsNames();
        int columnSize = names.length;
        ArrayList row = RowsUtils.getCurrentTuple(rows);

        for(int i = 0; i < columnSize; ++i) {
            map.put(names[i], row.get(i));
        }

        return map;
    }

    public synchronized HashMap transformIntoHashMapWithoutNull() throws SQLException {
        HashMap map = new CaseInsensitiveHashMap();
        String[] names = getColumnsNames();
        int columnSize = names.length;
        ArrayList row = RowsUtils.getCurrentTuple(rows);

        for(int i = 0; i < columnSize; ++i) {
            Object o = row.get(i);
            if(o == null)o = "";

            map.put(names[i], o);
        }

        return map;
    }

    /**
     *  Ű row key:columns name , value:columns data ־ map Ƽ Ѵ.
     * @param map
     * @return
     * @throws SQLException
     */
    public synchronized Map transformIntoMapOn(Map map) throws SQLException {
        String[] names = getColumnsNames();
        int columnSize = names.length;
        ArrayList row = RowsUtils.getCurrentTuple(rows);

        for(int i = 0; i < columnSize; ++i) {
            map.put(names[i], row.get(i));
        }

        return map;
    }

    /**
     * ü row vector  array 
     *
     * @return
     * @throws SQLException
     */
    public synchronized Vector[] transformIntoVectors() throws SQLException {
        return transformIntoVectors(this.rows);
    }

    public static Vector[] transformIntoVectors(RowSet rs) throws SQLException {
        Vector vs = new Vector();

        rs.beforeFirst();
        while(rs.next()) {
            ArrayList row = RowsUtils.getCurrentTuple(rs);
            vs.add(row);
        }
        rs.beforeFirst();
        return(Vector[])vs.toArray(new Vector[0]);
    }

    /**
     *  Ű row list 
     *
     * @return
     * @throws SQLException
     */
    public synchronized ArrayList transformIntoArrayList() throws SQLException {
        return RowsUtils.getCurrentTuple(rows);
    }

    public static ArrayList transformIntoArrayList(RowSet rs) throws SQLException {
        return RowsUtils.getCurrentTuple(rs);
    }

    /**
     * Ѱ row bean Ƽ ü row 迭 
     * bean property ̸ row columns name  ؼ   ش property  Ѵ.
     *
     * <p>! : columns  type property type ¾ƾ Ѵ.
     *   ڿ   property type String̾ ϸ ¥  java.util.Date̾ Ѵ.
     *    int,float,double... ̾ Ѵ.
     * ڼ  java.sql.ResultSet.getObject()   .
     * </p>
     *
     * @param beanClass  bean class
     * @return
     * @throws SQLException
     * @throws InstanceCreationException
     *
     * @see java.sql.ResultSet
     */
    public synchronized Object[] transformIntoBeans(Class beanClass) throws SQLException, InstanceCreationException {
        return transformIntoBeans(this.rows, beanClass);
    }

    public static Object[] transformIntoBeans(RowSet rs, Class beanClass) throws SQLException, InstanceCreationException {
        Vector objects = new Vector();

        String[] names = getColumnsNames(rs);
        int columnSize = names.length;

        rs.beforeFirst();
        while(rs.next()) {
            Object object = InstanceCreator.createBeanWithPropertyValues(beanClass, names, RowsUtils.getCurrentTuple(rs).toArray());
            objects.add(object);
        }
        return objects.toArray((Object[])Array.newInstance(beanClass, 0));
    }

    /**
     * @param rs
     * @param beanClass
     * @return null if the first row does not exist
     * @throws SQLException
     * @throws InstanceCreationException
     */
    public static Object transformFirstRowIntoBean(RowSet rs, Class beanClass) throws SQLException, InstanceCreationException {
        rs.beforeFirst();
        if(rs.next()) {
            String[] names = getColumnsNames(rs);
            return InstanceCreator.createBeanWithPropertyValues(beanClass, names, RowsUtils.getCurrentTuple(rs).toArray());
        } else return null;
    }

    /**
     *  õ row bean Ƽ 
     *
     * @param beanClass  bean class
     * @return
     * @throws SQLException
     * @throws InstanceCreationException
     *
     * @see transeformBeans()
     */
    public synchronized Object transformIntoBean(Class beanClass) throws SQLException, InstanceCreationException {
        String[] names = getColumnsNames();
        return InstanceCreator.createBeanWithPropertyValues(beanClass, names, RowsUtils.getCurrentTuple(rows).toArray());
    }

    /**
     * @param beanObject row  property   object
     * @return
     * @throws SQLException
     * @throws InstanceCreationException
     */
    public synchronized Object transformIntoBeanOn(Object beanObject) throws SQLException, InstanceCreationException {
        String[] names = getColumnsNames();
        return InstanceCreator.setPropertyValues(beanObject, names, RowsUtils.getCurrentTuple(rows).toArray());
    }

}
