package com.bleujin.framework.db.h2;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.RepositoryException;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.procedure.Queryable;
import com.bleujin.framework.db.procedure.UserProcedure;
import com.bleujin.framework.db.procedure.UserProcedures;
import com.bleujin.framework.util.StringUtil;

public class H2ChainUserProcedure extends UserProcedure {

	private String procName;
	private String chainedSQL;
	private UserProcedures chain;
	private ArrayList querys = new ArrayList();

	protected H2ChainUserProcedure(IDBController dc, String procName, String stmtSQL) {
		super(dc, procName);
		this.procName = procName;
		this.chainedSQL = stmtSQL;
		this.chain = makeChainProcedure(stmtSQL);

	}

	private UserProcedures makeChainProcedure(String stmtSQL) {
		UserProcedures chain = getDBController().createUserProcedures(procName) ;
		String[] queryStr = StringUtil.split(stmtSQL, ";");
		for (int i = 0; i < queryStr.length; i++) {
			if (StringUtil.isBlank(queryStr[i])) continue ;
			H2UserProcedure cmd = (H2UserProcedure)getDBController().createParameterQuery(queryStr[i]);
			cmd.addNamedParam(this.getNamedParam());

			chain.add(cmd);
		}
		return chain ;
	}

	@Override
	protected Rows myQuery(Connection conn) throws SQLException {
		return chain.execQuery();
	}

	final String ParamPattern = "\\@\\{(\\w+)\\}";

	protected String getStmtSQL() {
		return transNamedSQL(this.chainedSQL);
	}

	protected int myUpdate(Connection conn) throws SQLException {
		return chain.execUpdate();
	}

	public Queryable getQuery(int i) {
		return (Queryable) querys.get(i);
	}

	private void setParam(PreparedStatement modifyps, int i) throws SQLException {
		int mediateLoc = i + 1; // return + start zero...

		if (isNull(i)) {
			if (((Integer) getTypes().get(i)).intValue() == Types.INTEGER) {
				modifyps.setNull(mediateLoc, Types.INTEGER);
			} else if (((Integer) getTypes().get(i)).intValue() == Types.FLOAT) {
				modifyps.setNull(mediateLoc, Types.FLOAT);
			} else if (((Integer) getTypes().get(i)).intValue() == Types.DATE) {
				modifyps.setNull(mediateLoc, Types.DATE);
			} else {
				modifyps.setNull(mediateLoc, Types.VARCHAR);
			}
			// modifyps.setNull();
		} else {
			modifyps.setObject(mediateLoc, getParams().get(i));
		}
	}

	protected Statement getStatement() {
		throw RepositoryException.throwIt(new UnsupportedOperationException());
	}

}

