package com.bleujin.framework.db.h2;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Ref;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;

import com.bleujin.framework.db.ExtendTransformer;
import com.bleujin.framework.db.IONXmlWriter;
import com.bleujin.framework.db.IXmlWriter;
import com.bleujin.framework.db.Page;
import com.bleujin.framework.db.RepositoryException;
import com.bleujin.framework.db.Row;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.RowsUtils;
import com.bleujin.framework.db.bean.ResultSetHandler;
import com.bleujin.framework.db.procedure.IQueryable;
import com.bleujin.framework.db.rowset.WebRowSet;
import com.bleujin.framework.util.StringUtil;

public class H2Rows extends WebRowSet implements Rows {

	private static final long serialVersionUID = 646563401686178061L;

	private Rows nextRows = null;
	private IXmlWriter xmlWriter = null;
	private IQueryable query ;

	H2Rows(IQueryable query) throws SQLException {
		super();
		this.query = query ;
		setXmlWriter(new IONXmlWriter());
	}

	public String getString(int i) throws SQLException {
		return super.getString(i);
		// return getClobString(i);
	}

	public InputStream getBinaryStream(int i) throws SQLException {
		Object obj = super.getBinaryStream(i) ; 
		if (obj == null) {
			return null;
		} else {
			// System.out.println("obj................ : " + obj.getClass().getName()) ;
			return (InputStream) obj;
			// binaryStream = new ByteArrayInputStream((byte[]) (byte[]) obj);
			// return binaryStream;
		}
	}

	private String getClobString(int i) throws SQLException {
		ResultSetMetaData meta = this.getMetaData();
		if (meta.getColumnType(i) == Types.CLOB) {
			if (this.getClob(i) == null)
				return StringUtil.EMPTY;
			else {
				return RowsUtils.clobToString(this.getClob(i));
			}
		} else {
			return super.getString(i);
		}
	}

	public String getString(String column) throws SQLException {
		return super.getString(column);
		// return getClobString(column);
	}

	public Object getObject(int i) throws SQLException {
		return getClobObject(i);
	}

	private Object getClobObject(int i) throws SQLException {
		ResultSetMetaData meta = this.getMetaData();
		if (meta.getColumnType(i) == Types.CLOB) {
			if (this.getClob(i) == null)
				return StringUtil.EMPTY;
			else
				return RowsUtils.clobToString(this.getClob(i));
		} else {
			return super.getObject(i);
		}
	}

	public Object getObject(String column) throws SQLException {
		return getClobObject(getColumnIndex(column));
	}

	public boolean getBoolean(int i) throws SQLException {
		ResultSetMetaData meta = this.getMetaData();

		if (meta.getColumnType(i) == Types.VARCHAR || meta.getColumnType(i) == Types.CHAR) {
			if (super.getString(i) == null)
				return false;
			return (StringUtil.T.equals(super.getString(i).toUpperCase()) || StringUtil.TRUE.equals(super.getString(i).toUpperCase())) ? true : false;
		} else {
			return super.getBoolean(i);
		}
	}

	public boolean getBoolean(String column) throws SQLException {
		return getBoolean(getColumnIndex(column));
	}

	private int getColumnIndex(String column) throws SQLException {
		ResultSetMetaData meta = this.getMetaData();
		for (int i = 1, last = meta.getColumnCount(); i <= last; i++) {
			if (meta.getColumnName(i).toUpperCase().equals(column.toUpperCase())) {
				return i;
			}
		}
		throw new SQLException("Not Valid Column Name. " + column + " Not Exists");
	}

	// private Map nameToIndexMap = null;
	private String getClobString(String column) throws SQLException {
		ResultSetMetaData meta = this.getMetaData();
		int i = getColumnIndex(column);

		if (meta.getColumnType(i) == Types.CLOB) {
			return this.getClobString(i);
		} else {
			return super.getString(i);
		}
	}

	public int getRowCount() {
		try {
			int result = 0;
			int currLoc = super.getRow();
			if (!last()) {
				return 0;
			}
			result = super.getRow();

			if (currLoc > 0) {
				absolute(currLoc);
			} else {
				beforeFirst();
			}
			return result;
		} catch (SQLException ex) {
			throw RepositoryException.throwIt(ex);
		}
	}

	public Rows setNextRows(Rows rows) {
		this.nextRows = rows;
		return rows;
	}

	public Rows getNextRows() {
		return nextRows;
	}

	public void setXmlWriter(IXmlWriter writer) throws SQLException {
		this.xmlWriter = writer;
	}

	public void writeXml(Writer writer, IXmlWriter xmlWriter) throws SQLException {
		if (xmlWriter != null) {
			setXmlWriter(xmlWriter);
		}
		writeXml(writer);
	}

	public IXmlWriter getIXmlWriter() throws SQLException {
		return this.xmlWriter;
	}

	public String toString() {
		Writer writer = new StringWriter();
		try {
			this.writeXml(writer);
			writer.flush();
			writer.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return writer.toString();
	}

	public URL getURL(int columnIndex) {
		throw new NoSuchMethodError();
	}

	public void updateArray(int columnIndex, Array x) {
		throw new NoSuchMethodError();
	}

	public void updateBlob(int columnIndex, Blob x) {
		throw new NoSuchMethodError();
	}

	public void updateClob(int columnIndex, Clob x) {
		throw new NoSuchMethodError();
	}

	public void updateRef(int columnIndex, Ref x) {
		throw new NoSuchMethodError();
	}

	public URL getURL(String columnName) {
		throw new NoSuchMethodError();
	}

	public void updateArray(String columnName, Array x) {
		throw new NoSuchMethodError();
	}

	public void updateBlob(String columnName, Blob x) {
		throw new NoSuchMethodError();
	}

	public void updateClob(String columnName, Clob x) {
		throw new NoSuchMethodError();
	}

	public void updateRef(String columnName, Ref x) {
		throw new NoSuchMethodError();
	}

	public String getDefaultString(String col1, String defaultString) throws SQLException {
		String str = this.getString(col1);
		return (str == null || "".equals(str)) ? defaultString : col1;
	}

	public Row firstRow() {
		return ExtendTransformer.fetchFirstToRow(this) ;
    }

	public Rows nextPageRows() throws SQLException{
		Page nextPage = query.getPage().getNextPage() ; 
		query.setPage(nextPage) ;
		return query.execQuery() ;
	}
	
	public Rows prePageRows() throws SQLException {
		query.setPage(query.getPage().getPrePage()) ;
		return query.execQuery() ;
		
	}
	
	public Object toHandle(ResultSetHandler rsh) throws SQLException {
		return rsh.handle(this);
	}

}
