package com.bleujin.framework.db.h2;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.procedure.UserProcedure;

public class H2UserProcedure extends UserProcedure {
	private PreparedStatement pstmt;

	private String stmtSQL;

	protected H2UserProcedure(IDBController dc, String procName, String stmtSQL) {
		super(dc, procName);
		this.stmtSQL = stmtSQL;
	}


	@Override
	protected Rows myQuery(Connection conn) throws SQLException {
		Rows result = null ;
		ResultSet rs = null;

		try {
			pstmt = conn.prepareStatement(getStmtSQL(), ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);

			pstmt.setMaxRows(getMaxRows());
			pstmt.setFetchSize(getFetchSize());

			for (int i = 0; i < getParams().size(); i++) {
				setParam(pstmt, i);
			}
			rs = pstmt.executeQuery();
			result = super.populate(rs);
		} catch (Exception ex) {
			ex.printStackTrace();
			throw new SQLException(ex.getMessage());
		} finally {
        	closeSilence(rs, pstmt, conn) ;
		}
		return result;
	}

	final String ParamPattern = "\\@\\{(\\w+)\\}";

	protected String getStmtSQL() {
		return transNamedSQL(this.stmtSQL);
	}

	protected int myUpdate(Connection conn) throws SQLException {
		int result = 0;
		try {
			pstmt = conn.prepareCall(getStmtSQL());

			for (int i = 0; i < getParams().size(); i++) {
				setParam(pstmt, i);
			}

			result = pstmt.executeUpdate();

			pstmt.close();
		} catch (SQLException ex) {
			throw ex;
		} finally {
			closeSilence(pstmt) ;
		}

		return result;
	}

	private void setParam(PreparedStatement modifyps, int i) throws SQLException {
		int mediateLoc = i + 1; // return + start zero...

		if (isNull(i)) {
			if (((Integer) getTypes().get(i)).intValue() == Types.INTEGER) {
				modifyps.setNull(mediateLoc, Types.INTEGER);
			} else if (((Integer) getTypes().get(i)).intValue() == Types.FLOAT) {
				modifyps.setNull(mediateLoc, Types.FLOAT);
			} else if (((Integer) getTypes().get(i)).intValue() == Types.DATE) {
				modifyps.setNull(mediateLoc, Types.DATE);
			} else {
				modifyps.setNull(mediateLoc, Types.VARCHAR);
			}
			// modifyps.setNull();
		} else {
			modifyps.setObject(mediateLoc, getParams().get(i));
		}
	}

	protected Statement getStatement() {
		return pstmt;
	}

}
