package com.bleujin.framework.db.hsql;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.xml.sax.SAXException;

import com.bleujin.framework.db.RepositoryException;
import com.bleujin.framework.db.manager.DBManager;
import com.bleujin.framework.db.manager.DBType;
import com.bleujin.framework.db.procedure.HSQLBean;
import com.bleujin.framework.db.procedure.HSQLExtendRepositoryService;
import com.bleujin.framework.db.procedure.HSQLParser;
import com.bleujin.framework.db.procedure.RepositoryService;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * <p>
 * Company: i-on
 * </p>
 * 
 * @author bleujin
 * @version 1.0
 */

public class HSQLDBManager extends DBManager {

	private final static String DRIVER_NAME = "org.hsqldb.jdbcDriver";

	private final RepositoryService service;
	private final HSQLBean bean;

	public HSQLDBManager(HSQLBean bean) {
		this.bean = bean;
		this.service = new HSQLExtendRepositoryService(bean);
	}

	public HSQLDBManager(String configFile) {
		try {
			HSQLParser parser = new HSQLParser(configFile);
			HSQLBean bean = parser.getHQLBean();
			this.bean = bean;
			this.service = new HSQLExtendRepositoryService(bean);
		} catch (SAXException e) {
			throw RepositoryException.throwIt(e);
		} catch (IOException e) {
			throw RepositoryException.throwIt(e);
		}
	}

	protected void myDestroyPool() throws SQLException {
	}

	public Connection getConnection() throws SQLException {
		return DriverManager.getConnection(getJdbcURL(), getUserId(), getUserPwd());
	}

	public int getDBManagerType() {
		return DBType.HSQLDBManager;
	}

	public String getDBType() {
		return DBType.HSQLDBName;
	}

	public RepositoryService getRepositoryService() {
		return service;
	}

	protected void myInitPool() throws SQLException {
		try {
			Class.forName(getDriverName());
		} catch (ClassNotFoundException ex) {
			throw new SQLException(ex.getMessage());
		}
	}

	public String getJdbcURL() {
		return bean.getAddress();
	}

	public String getDriverName() {
		return DRIVER_NAME;
	}

	public String getUserPwd() {
		return bean.getUserPwd();
	}

	public String getUserId() {
		return bean.getUserId();
	}
}
