package com.bleujin.framework.db.manager;

import java.sql.Connection;
import java.sql.SQLException;

import oracle.jdbc.pool.OracleConnectionCacheImpl;
import oracle.jdbc.pool.OracleConnectionPoolDataSource;

/**
 * <p>Title: EV</p>
 * <p>Description: I-ON Content Server</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: I-ON Communications</p>
 * @author bleujin
 * @version 1.0
 */

public class Oracle9iCacheDBManager extends OracleCacheDBManager {
    private OracleConnectionCacheImpl ocacheimpl = null;
    private OracleConnectionPoolDataSource cpds = null;
    final private int MAX_TRY = 5 ;
    private int initConnectionCount = 10 ;

    public Oracle9iCacheDBManager() {
        super("jdbc:oracle:thin:@ics4db:1521:DB9I", "odin", "odin");
    }

    public Oracle9iCacheDBManager(String jdbcURL, String user, String passwd) {
        this(jdbcURL, user, passwd, 20);
    }

    public Oracle9iCacheDBManager(String jdbcURL, String user, String passwd, int connectionLimit) {
        super(jdbcURL, user, passwd, connectionLimit);
    }

    protected synchronized void myInitPool() throws SQLException {
        // to resolve a rowset bug; http://developer.java.sun.com/developer/bugParade/bugs/4625851.html
        // NumberFormatException
        System.setProperty("oracledatabasemetadata.get_lob_precision", "false");
        this.initializeConnectionCacheDataSrc();

        // Initialize the Connection Cache
        ocacheimpl = new OracleConnectionCacheImpl(cpds);
        ocacheimpl.setMaxLimit(this.initConnectionCount <= 3 ? 3 : this.initConnectionCount);
        ocacheimpl.setMinLimit(this.initConnectionCount / 4 <= 3 ? 3 : this.initConnectionCount / 4);

        // Set Caching Scheme as DYNAMIC_SCHEME
        // Caching Schema means that once the connection active size becomes 5,
        // the next request for Connection will be served by creating a new pooled
        // connection instance and close the connection automatically when it is
        // no longer in use.
        ocacheimpl.setCacheScheme(OracleConnectionCacheImpl.DYNAMIC_SCHEME);
    }

    private void initializeConnectionCacheDataSrc() throws SQLException {
        // Initialize the Datasource
        cpds = new OracleConnectionPoolDataSource();

        // Configure the Datasource with proper values of Host Name, User Name etc
        cpds.setDriverType("thin");
        cpds.setNetworkProtocol("tcp");
        cpds.setURL(getJdbcURL());

        cpds.setUser( getUserId());
        cpds.setPassword( getUserPwd());
    }

    public int getActiveSize() {
      return ocacheimpl.getActiveSize();
    }

    public int getCacheSize() {
       return ocacheimpl.getCacheSize();
    }

    protected synchronized void myDestroyPool() throws SQLException {
        if(ocacheimpl != null) {
        	ocacheimpl.close();
        }
    }


    public Connection getConnection() throws SQLException {
        return ocacheimpl.getConnection();
    }



    public int getDBManagerType() {
        return DBType.Oracle9iCacheDBManager;
    }
}
