package com.bleujin.framework.db.manager;

import java.sql.Connection;
import java.sql.SQLException;

public class OracleGenericPoolDBManager extends OracleDBManager {

	private PoolHelper pHelper;
	private int maxLimit = 40;

	// Only For Test
	protected OracleGenericPoolDBManager() {
		super();
	}

	public OracleGenericPoolDBManager(String jdbcURL, String user, String passwd) {
		super(jdbcURL, user, passwd);
	}

	public OracleGenericPoolDBManager(String jdbcURL, String user, String passwd, int maxLimit) {
		super(jdbcURL, user, passwd);
		this.maxLimit = maxLimit;
	}

	protected void myInitPool() throws SQLException {
		try {
			Class.forName(getDriverName());
		} catch (ClassNotFoundException ex) {
			throw new SQLException("Driver Class Not Found", ex.toString());
		}
		pHelper = new PoolHelper(getJdbcURL(), getUserId(), getUserPwd(), maxLimit);
		pHelper.initPoolConnection();
		// to resolve a rowset bug;
		// http://developer.java.sun.com/developer/bugParade/bugs/4625851.html
		// NumberFormatException
		System.setProperty("oracledatabasemetadata.get_lob_precision", "false");
	}

	public Connection getConnection() throws SQLException {
		return pHelper.getConnection();
		// return pc.getConnection();
	}

	public void freeConnection(Connection conn) throws SQLException {
		pHelper.freeConnection(conn);
	}

	protected void myDestroyPool() throws SQLException {
		pHelper.destroyPoolConnection();
	}

	public int getDBManagerType() {
		return DBType.OraclePoolDBManager;
	}
}
