package com.bleujin.framework.db.procedure;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.bean.ResultSetHandler;
import com.bleujin.framework.db.manager.DBManager;
import com.bleujin.framework.db.manager.DBType;


/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: I-ON Communications</p>
 * @author bleujin
 * @version 1.0
 */

public class CombinedUserProcedures extends Queryable
{

    private List<Query> querys = new ArrayList<Query>();
    private IQueryable currQuery = null ;

    public CombinedUserProcedures(IDBController dc, String name) {
        super(dc, name, QueryType.USER_PROCEDURES);
    }

    public CombinedUserProcedures add(String strSQL, String name, int type) {
        add(getDBController().createParameterQuery(strSQL), name, type);
        return this;
    }

    public CombinedUserProcedures add(IQueryable query) {
        return this.add(query, query.getProcSQL(), UPDATE_COMMAND) ;
    }

    public CombinedUserProcedures add(IQueryable query, int type) {
        return this.add(query, query.getProcSQL(), type) ;
    }

    public CombinedUserProcedures add(IQueryable query, String name, int type) {
        if (this == query) throw new IllegalArgumentException("exception.framework.db.combined_procedures.infinite_loop") ;
        if ( ! (query instanceof Queryable)) throw new IllegalArgumentException("exception.framework.db.combined_procedures.not_supported_type") ;
        querys.add(new Query(query, name, type));
        return this;
    }

    protected Rows myQuery(Connection conn) throws SQLException {
        UserProcedures upts = getDBController().createUserProcedures(this.getProcSQL()) ;
        Query[] query = (Query[])querys.toArray(new Query[0]) ;
        for (int i = 0; i < query.length; i++) {
            upts.add(query[i].getQuery()) ;
        }
        return upts.execQuery();
    }

    private Map resultMap = new HashMap() ;
    protected int myUpdate(Connection conn) throws SQLException {

        for (int i = 0; i < querys.size(); i++) {
        	
            Query cquery = (Query)querys.get(i) ;
            Queryable query = cquery.getQuery();
            String name = cquery.getName();
            int query_type = cquery.getQueryType() ;

            setCurrentQuery(query) ;
            
            

            if (query_type == UPDATE_COMMAND){
                int updateRow = query.myUpdate(conn);
                resultMap.put(name, new Integer(updateRow)) ;
            } else if (query_type == QUERY_COMMAND) {
                Rows rows = query.myQuery(conn) ;
                resultMap.put(name, rows) ;
            } else {
            	throw new IllegalArgumentException(query_type + " is not defined type\n" + query) ;
            }
        }

        return querys.size();
    }



    public Map getResultMap(){
        return resultMap ;
    }

    public Object getResult(Queryable query){
        return resultMap.get(query.getProcSQL()) ;
    }


    public IQueryable getQuery(int i) {
       return ((Query) querys.get(i)).getQuery();
   }
    
    protected Statement getStatement() {
    	if (currQuery instanceof Queryable){
            return ((Queryable)currQuery).getStatement() ;
    	} else {
    		throw new UnsupportedOperationException("Unknown Query Type : ") ;
    	}
    }

    private void setCurrentQuery(IQueryable query){
        this.currQuery = query ;
    }

}

class Query {

    private IQueryable query ;
    private String name ;
    private int query_type ;
    private ResultSetHandler handler ;

    public Query(IQueryable query, String name, int query_type){
        this(query, name, query_type, null) ;
    }

    public Query(IQueryable query, String name, int query_type, ResultSetHandler handler){
        this.query = query ;
        this.name = name ;
        this.query_type = query_type ;
        this.handler = handler ;
    }

    Queryable getQuery(){
		if (! (query instanceof Queryable)) 
			throw new IllegalArgumentException("exception.framework.db.userporcedures.not_supported_query_type");

        return (Queryable)query ;
    }

    String getName(){
        return name ;
    }

    int getQueryType(){
        return query_type ;
    }

    ResultSetHandler getHandler(){
        return handler ;
    }
}

class InnerDBManager extends DBManager {

    private DBManager dbm ;
    private Connection conn ;

    InnerDBManager(DBManager dbm, Connection conn) {
        this.dbm = dbm ;
        this.conn = conn ;
    }

    public int getDBManagerType(){
        return DBType.UNKNOWN ;
    }
    public String getDBType(){
        return DBType.UnknownDBName ;
    }
    public RepositoryService getRepositoryService() {
        try {
            return RepositoryService.makeService(this.conn);
        } catch(SQLException ex) {
            throw new IllegalArgumentException(ex.getMessage()) ;
        }
    }

    public Connection getConnection() {
        return this.conn ;
    }

    public void freeConnection(Connection conn) throws SQLException {
    }

    protected void myInitPool(){
    }

    protected void myDestroyPool() throws SQLException{
    }

}
