package com.bleujin.framework.db.procedure;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.RepositoryException;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.manager.DBManager;
import com.bleujin.framework.db.manager.DBType;
import com.bleujin.framework.util.StringUtil;


public class HSQLChainUserProcedure extends UserProcedure {

	private String procName;
	private String chainedSQL;
	private UserProcedures chain;
	private ArrayList<IQueryable> querys = new ArrayList<IQueryable>();

	protected HSQLChainUserProcedure(IDBController dc, String procName, String stmtSQL) {
		super(dc, procName);
		this.procName = procName;
		this.chainedSQL = stmtSQL;
		this.chain = new DBController(dc.getDBManager()).createUserProcedures(stmtSQL);
		addProcedure(stmtSQL, chain);

	}

	private void addProcedure(String stmtSQL, UserProcedures chain) {
		String[] queryStr = StringUtil.split(stmtSQL, ";");
		for (int i = 0; i < queryStr.length; i++) {
			HSQLUserProcedure cmd = new HSQLUserProcedure(getDBController(), procName + "_" + i, queryStr[i]);
			cmd.addNamedParam(this.getNamedParam());

			chain.add(cmd);
		}
	}

	@Override
	protected Rows myQuery(Connection conn) throws SQLException {
		return chain.execQuery();
	}

	final String ParamPattern = "\\@\\{(\\w+)\\}";

	protected String getStmtSQL() {
		return transNamedSQL(this.chainedSQL);
	}

	protected int myUpdate(Connection conn) throws SQLException {
		return chain.myUpdate(conn);
	}

	public Queryable getQuery(int i) {
		return (Queryable) querys.get(i);
	}

	protected Statement getStatement() {
		throw RepositoryException.throwIt(new UnsupportedOperationException());
	}

}

class InnerHSQLDBManager extends DBManager {

	private DBManager dbm;
	private Connection conn;

	InnerHSQLDBManager(DBManager dbm, Connection conn) {
		this.dbm = dbm;
		this.conn = conn;
	}

	public int getDBManagerType() {
		return DBType.UNKNOWN;
	}

	public String getDBType() {
		return DBType.UnknownDBName;
	}

	public RepositoryService getRepositoryService() {
		try {
			return RepositoryService.makeService(this.conn);
		} catch (SQLException ex) {
			throw new IllegalArgumentException(ex.getMessage());
		}
	}

	public Connection getConnection() {
		return this.conn;
	}

	public void freeConnection(Connection conn) throws SQLException {
	}

	protected void myInitPool() {
	}

	protected void myDestroyPool() throws SQLException {
	}

}