package com.bleujin.framework.db.procedure;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import junit.framework.Assert;

import com.bleujin.framework.db.IDBController;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.RowsUtils;
import com.bleujin.framework.util.CaseInsensitiveHashMap;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author bleujin
 * @version 1.0
 */

public abstract class ParameterQueryable extends Queryable implements IParameterQueryable {

	protected List params = new ArrayList();
	protected List types = new ArrayList();
	protected Map namedParams = new CaseInsensitiveHashMap();

	private int idx = 0;

	public ParameterQueryable(IDBController dc, String procSQL, int queryType) {
		super(dc, procSQL, queryType);
	}


	protected final Rows populate(ResultSet rs) throws SQLException {
		Rows result = RowsUtils.create(this);
		result.populate(rs, getPage().getStartLoc(), getPage().getListNum());
		return result;
	}
    
	public IParameterQueryable addParam(boolean param) {
		addParam(idx++, param);
		return this;
	}

	public IParameterQueryable addParam(int param) {
		addParam(idx++, param);
		return this;
	}

	public IParameterQueryable addParam(long param) {
		addParam(idx++, param);
		return this;
	}

	public IParameterQueryable addParam(Object param) {
		addParam(idx++, param);
		return this;
	}

	public IParameterQueryable addParam(String param) {
		addParam(idx++, param);
		return this;
	}

	public IParameterQueryable addClob(String param) {
		addClob(idx++, param);
		return this;
	}

	// .................
	public IParameterQueryable addParam(String name, boolean obj) {
		namedParams.put(name, new ParamValue(new Boolean(obj), Types.BOOLEAN));
		return this;
	}

	public IParameterQueryable addParam(String name, int obj) {
		namedParams.put(name, new ParamValue(new Integer(obj), Types.INTEGER));
		return this;
	}

	public IParameterQueryable addParam(String name, long obj) {
		namedParams.put(name, new ParamValue(new Long(obj), Types.BIGINT));
		return this;
	}

	public IParameterQueryable addParam(String name, Object param) {
		if (param instanceof IStringObject) {
			return addParam(name, ((IStringObject) param).getString()) ;
		} else {
			namedParams.put(name, new ParamValue(param, Types.OTHER));
			return this;
		}
	}

	public IParameterQueryable addParam(String name, String param) {
		namedParams.put(name, new ParamValue(param, Types.VARCHAR));
		return this;
	}

    public IParameterQueryable addParam( String name, Object param, int type) {
    	namedParams.put(name, new ParamValue(param, type));
    	return this ;
    }
    public IParameterQueryable addClob( String name, String param) {
    	namedParams.put(name, new ParamValue(param, Types.CLOB));
    	return this ;
    }
    public IParameterQueryable addBlob( String name, InputStream param) {
    	namedParams.put(name, new ParamValue(param, Types.BLOB));
    	return this ;
    }

	
	
	public Map getNamedParam() {
		return Collections.unmodifiableMap(namedParams);
	}

	public IParameterQueryable addNamedParam(Map namedParam) {
		this.namedParams = namedParam;
		return this;
	}

	class ParamValue {
		ParamValue(Object value, int type) {
			this.value = value;
			this.type = type;
		}

		Object value;

		int type;
		
		public String toString(){
			return value == null ? null : value.toString() ;
		}

	}

	protected final String patternStr = "\\:[a-zA-Z][a-zA-Z0-9]*";

	public String getProcSQL() {
		return transNamedSQL(super.getProcSQL());
	}

	protected String transNamedSQL(String _sql) {
		if (namedParams.isEmpty()) {
			return _sql;
		} else {
			String baseSQL = _sql;
			// when select ... from table wheren col1 = :name1 and col2 = :name2
			// and col3 = (:name1 + :name2)
			// list will contain names of: name1, name2, name1, name2
			List list = getSettedParam(baseSQL);
			ParamValue[] tempParam = new ParamValue[list.size()];

			
			for (int i = 0, last = list.size(); i < last; i++) {
				ParamValue paramValue = (ParamValue) namedParams.get(list.get(i));
				
				if (paramValue == null) throw new IllegalArgumentException("unsetted named param : " + list.get(i)) ;
				tempParam[i] = paramValue;
			}
			// now, we have tempParam list which contains values for the list.
			for (int i = 0; i < tempParam.length; i++) {
				this.params.add(tempParam[i].value);
				this.types.add(tempParam[i].type);
			}
			return baseSQL.replaceAll(patternStr, "?");
		}
	}

	protected List getSettedParam(String baseSQL) {
		List result = new ArrayList();
		Pattern pattern = Pattern.compile(patternStr);
		Matcher matcher = pattern.matcher(baseSQL);

		while (matcher.find()) {
			result.add(baseSQL.substring(matcher.start() + 1, matcher.end()));
		}
		return result;
	}

	public void addParam(int paramindex, boolean obj) {
		addParam(paramindex, new Boolean(obj), Types.BOOLEAN);
	}

	public void addParam(int paramindex, int obj) {
		addParam(paramindex, new Integer(obj), Types.INTEGER);
	}

	public void addParam(int paramindex, long obj) {
		addParam(paramindex, new Long(obj), Types.BIGINT);
	}

	public void addParam(int paramindex, Object param) {
		addParam(paramindex, param, Types.OTHER);
	}

	public void addParam(int paramindex, String param) {
		addParam(paramindex, param, Types.VARCHAR);
	}

	public void addParam(int paramindex, Object param, int type) {
		if (type == Types.OTHER && param instanceof IStringObject){
			addParam(paramindex, (param==null) ? param : ((IStringObject)param).getString(), type) ;
		} else {
			params.add(paramindex, param);
			types.add(paramindex, new Integer(type));
		}
	}

	// about lob...
	public void addClob(int paramindex, String clobString) {
		addParam(paramindex, clobString, Types.CLOB);
	}

	public void addBlob(int paramindex, String fileName) throws FileNotFoundException {
		addParam(paramindex, new FileInputStream(new File(fileName)), Types.BLOB);
	}

	public void addBlob(int paramindex, InputStream is) {
		addParam(paramindex, is, Types.BLOB);
	}

	public boolean isNull(int paramIndex) {
		return params.get(paramIndex) == null;
	}

	boolean isClobType(int paramindex) {
		return (new Integer(Types.CLOB)).equals(types.get(paramindex));
	}

	boolean isBlobType(int paramindex) {
		return (new Integer(Types.BLOB)).equals(types.get(paramindex));
	}

	int getBlobParamCount() {
		int result = 0;
		for (int i = 0, last = types.size(); i < last; i++) {
			if (isBlobType(i))
				result++;
		}
		return result;
	}

	int getClobParamCount() {
		int result = 0;
		for (int i = 0, last = types.size(); i < last; i++) {
			if (isClobType(i))
				result++;
		}
		return result;
	}

	int getClobParamIndex(int keyIndex) {
		int currIndex = 0;
		for (int i = 0, last = types.size(); i < last; i++) {
			if (isClobType(i)) {
				if (currIndex == keyIndex)
					return i;
				else
					currIndex++;
			}
		}
		throw new IllegalArgumentException("Not Found " + keyIndex + "'th Clob Type");
	}

	int getBlobParamIndex(int keyIndex) {
		int currIndex = 0;
		for (int i = 0, last = types.size(); i < last; i++) {
			if (isBlobType(i)) {
				if (currIndex == keyIndex)
					return i;
				else
					currIndex++;
			}
		}
		throw new IllegalArgumentException("Not Found " + keyIndex + "'th Blob Type");
	}

	boolean hasLobType() {
		return (getClobParamCount() + getBlobParamCount()) > 0;
	}

	// end lob type...

	public void setParamValues(List params, List types) {
		Assert.assertNotNull("Parameter Map is Null", params);
		Assert.assertNotNull("Types Map is Null", types);

		this.params = params;
		this.types = types;
	}

	public String getProcFullSQL() {
		StringBuffer result = new StringBuffer(getProcSQL() + "\n");
		for (int i = 0; i < params.size(); i++) {
			result.append(String.valueOf(i) + "(" + types.get(i) + ") : " + params.get(i) + "\n");
		}
		return result.toString();
	}

	public String[] getParamsAsString(int index) {
		Object obj = params.get(index);
		if (obj == null)
			return new String[] { "" };
		else if (obj instanceof Object[]) {
			Object[] objs = (Object[]) obj;
			String[] result = new String[objs.length];
			for (int i = 0; i < objs.length; i++) {
				result[i] = String.valueOf(objs[i]);
			}
			return result;
		} else
			return new String[] { String.valueOf(obj) };
	}

	public String getParamAsString(int index) {
		return getParamsAsString(index)[0];
	}

	public List getParams() {
		return params;
	}

	public List getTypes() {
		return types;
	}

	public int getType(int index) {
		return ((Integer) types.get(index)).intValue();
	}
	
	public void clearParam() {
		params.clear() ;
		types.clear() ;
		namedParams.clear() ;
		idx = 0 ;
	}
}
