package com.bleujin.framework.db.procedure;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;

import com.bleujin.framework.db.IDBController;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author bleujin
 * @version 1.0
 */

public abstract class RepositoryService {

    public final static RepositoryService ORACLE = new OracleRepositoryService() ;
    public final static RepositoryService MSSQL = new MSSQLRepositoryService() ;


    private final static String PROCEDURE_PATTERN = ".+\\@.+\\(.*[\\?\\,]*\\)" ;
    private final static String PROCEDURE_SHORT_PATTERN = "[a-zA-Z0-9]+\\@[a-zA-Z0-9]+" ;
    protected RepositoryService() {
    }

    public abstract IUserProcedureBatch createUserProcedureBatch(IDBController dc, String procSQL);
    public abstract IUserCommand createUserCommand(IDBController dc, String sql) ;
    public abstract IUserCommandBatch createUserCommandBatch(IDBController dc, String sql) ;
    public abstract IUserProcedure createUserProcedure(IDBController dc, String proc);

    
    public UserProcedures createUserProcedures(IDBController dc, String name){
        return new UserProcedures(dc, name) ;
    }

    public CombinedUserProcedures createCombinedUserProcedures(IDBController dc, String name){
        return new CombinedUserProcedures(dc, name) ;
    }

    public IParameterQueryable createParameterQuery(IDBController dc, String strSQL) {
        if (isProcedurePattern(strSQL)) return createUserProcedure(dc, strSQL) ;
        else return createUserCommand(dc, strSQL) ;
    }

    private boolean isProcedurePattern(String strSQL) {
    	return isPattern(StringUtils.deleteWhitespace(strSQL), PROCEDURE_PATTERN)
    		|| isPattern(strSQL, PROCEDURE_SHORT_PATTERN) ;
    }

    private boolean isPattern(String strSQL, String patternStr) {
        Pattern pattern = Pattern.compile(patternStr);
        Matcher matcher = pattern.matcher(strSQL);

        return matcher.find(0) ;
    }

    public static RepositoryService makeService(Connection conn) throws SQLException {
        if ("ORACLE".equals(conn.getMetaData().getDatabaseProductName().toUpperCase())){
            return ORACLE ;
        } else {
            return MSSQL ;
        }
    }

}
