package com.bleujin.framework.parse.html;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import au.id.jericho.lib.html.Element;
import au.id.jericho.lib.html.Source;

import com.bleujin.framework.util.StringUtil;

public class HTag {

	private Element element ;
	private HTag(Element element){
		this.element = element ;
	}
	private HTag(Reader content, int index) throws IOException {
		this((Element)(new Source(content).getChildElements().get(index))) ;
	}
	
	static HTag create(Element element) throws IOException{
		return new HTag(element) ;
	}
	
	static HTag create(Reader content, int index) throws IOException{
		return new HTag(content, index) ;
	}
	
	public String getTagName() {
		return element.getName();
	}

	public String getTrimText() {
		return StringUtil.trim(element.getTextExtractor().toString());
	}

	public String getContent() {
		return element.toString();
	}

	public Element getElement(){
		return element ;
	}
	
	public String getAttributeValue(String attrName){
		return element.getAttributeValue(attrName) ;
	}
	
	public String getPath() {
		ArrayList<String> store = new ArrayList<String>() ;
		
		Element current = element ;
		Element parent = element.getParentElement() ;
		while(parent != null){
			List<Element> childs = parent.getChildElements() ;
			for (int i = 0, findex=0; i < childs.size(); i++) {
				if (childs.get(i).getName().equals(current.getName()) && current == childs.get(i)){ 
					store.add(current.getName() + "[" + findex + "]") ;
					break ;
				} else if (childs.get(i).getName().equals(current.getName())){
					findex++ ;
				}
			}
			
			current = parent ;
			parent = parent.getParentElement() ;
		}

		Collections.reverse(store) ;
		return "/" + StringUtil.join((String[])store.toArray(new String[0]), "/") ;
	}

	public boolean isExistChild() {
		return element.getChildElements().size() > 0;
	}

	public boolean isExistChild(String path) throws IOException{
		try {
			getChild(path) ;
		} catch(NotFoundTagException ex){
			return false ; 
		}
		return true ;
	}
	
	
	public HTag getFirstChild() throws IOException, NotFoundTagException {
		if (! isExistChild()) throw new NotFoundTagException("no child");
		
		return create((Element)element.getChildElements().get(0)) ; 
	}

	private Pattern pattern = Pattern.compile("(\\w*)\\[(\\d*)\\]");
	public HTag getChild(String path) throws IOException, NotFoundTagException {
		
		if (isRecursive(path)) {
			String beforePath = StringUtil.substringBefore(path + "/", "/") ;
			String afterPath = StringUtil.substringAfter(path, "/") ;
			HTag parent = getChild(beforePath) ;
			return parent.getChild(afterPath) ;
		}
		
		Matcher matcher = pattern.matcher(path);
		if (matcher.matches()){
			if (StringUtil.contains(path, "/")){
				return create(new StringReader("<font color=red>!!</font>"), 0) ;
			}
			return getChild(matcher.group(1), Integer.parseInt(matcher.group(2))) ;
		} else return getChild(path, 0) ;
	}

	private boolean isRecursive(String path){
		String afterPath = StringUtil.substringAfter(path, "/") ;
		return StringUtil.isNotBlank(afterPath) ;
	}
	
	
	public List<HTag> findElements(String type) throws IOException{
		List<HTag> result = new ArrayList<HTag>() ;
		List<Element> els = element.findAllElements(type) ;
		for (int i = 0, last=els.size(); i < last; i++) {
			result.add(create(els.get(i))) ;
		}
		
		return result ;
	}
	
	public HTag findElementById(String type, String idValue) throws IOException, NotFoundTagException {
		return findElementBy(type, "id", idValue) ;
	}

	public HTag findElementBy(String type, String attributeName, String attributeValue) throws IOException, NotFoundTagException {
		HTag result = null ;
		List<Element> els = element.findAllElements(type) ;
		for (int i = 0, last=els.size(); i < last; i++) {
			if (attributeValue.equals(els.get(i).getAttributeValue(attributeName))){
				result = new HTag(els.get(i)) ;
				break ;
			}
		}
		if (result == null) throw new NotFoundTagException("type:" +type + ",attributeName:" +attributeName + ",attributeValue:" +attributeValue) ;
		return result ;
	}

	public List<HTag> getChildren() throws IOException{
		return getChildren(0) ;
	}

	public List<HTag> getChildren(int skip) throws IOException{
		List<HTag> result = new ArrayList<HTag>() ;
		List<Element> childElement = element.getChildElements();
		for (int i = skip, last=childElement.size(); i < last; i++) {
			result.add(create(childElement.get(i))) ;
		}
		
		return result ;
	}
	

	
	public HTag getChild(String tagName, int positionIndex) throws NotFoundTagException, IOException {
		return getChild(this, tagName, positionIndex) ;
	}

	private HTag getChild(HTag parent, String tagName, int positionIndex) throws NotFoundTagException, IOException {
		List<HTag> children = parent.getChildren() ;
		int foundedIndex = 0 ;
		for (int i=0, last=children.size(); i < last ; i++) {
			HTag tag = children.get(i) ;
			if (tagName.equalsIgnoreCase(tag.getTagName())) { 
				if ( foundedIndex == positionIndex) {
					return create(tag.element) ;
				} else foundedIndex++ ;
			}
		}

		throw new NotFoundTagException("NOT FOUND : " + tagName);
	}

	public String[] findPathByText(String textValue) throws IOException {
		
		List<String> store = new ArrayList<String>() ;
		storePathByText(this, textValue, store, "") ; 
		
		return (String[])store.toArray(new String[0]) ;
	}
	
	private void storePathByText(HTag tag, String textValue, List<String> store, String prePath) throws IOException{
		
		List<HTag> children = tag.getChildren();
		if (textValue.trim().equals(tag.getTrimText())) {
			store.add(prePath + "/" + tag.getTagName()) ;
		}

		if(children.size() <= 0) return ;
		for (int i = 0, last=children.size(); i < last; i++) {
			storePathByText(children.get(i), textValue, store, prePath + "/" + tag.getTagName()) ;
		}
		
	}
	









	public int hashCode(){
		return toString().hashCode() ;
	}
	
	public boolean equals(Object that){
		if (that == null || !(that instanceof HTag)) return false ;
		return ( ((HTag)that).getContent() ).equals(this.getContent()) ;
	}
	
	public String toString(){
		return element.toString() ;
	}


}
