package com.bleujin.framework.tools;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.sql.ResultSet;
import java.sql.SQLException;

public class IOUtils {

	/**
	 * Close a reader without throwing an exception.
	 * 
	 * @param reader
	 *            the reader or null
	 */
	public static void closeSilently(Reader reader) {
		if (reader != null) {
			try {
				reader.close();
			} catch (IOException e) {
				// ignore
			}
		}
	}

	/**
	 * Close a writer without throwing an exception.
	 * 
	 * @param writer
	 *            the writer or null
	 */
	public static void closeSilently(Writer writer) {
		if (writer != null) {
			try {
				writer.flush();
				writer.close();
			} catch (IOException e) {
				// ignore
			}
		}
	}

	/**
	 * Close a result set without throwing an exception.
	 * 
	 * @param rs
	 *            the result set or null
	 */
	public static void closeSilently(ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				// ignore
			}
		}
	}

    /**
     * Convert an IO exception to a SQL exception.
     *
     * @param e the root cause
     * @param message the message
     * @return the SQL exception object
     */
    public static SQLException convertIOException(IOException e, String message) {
        if (message == null) {
            Throwable t = e.getCause();
            if (t != null && t instanceof SQLException) {
                return (SQLException) t;
            }
            return new SQLException(message + e.getMessage());
        }
        return new SQLException(message + e.getMessage());
    }

	public static IOException convertToIOException(SQLException e) {
		return new IOException(e.getMessage());
	}

}
