package com.bleujin.framework.util;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.TimeZone;

public class DateUtil {

	private static Hashtable dateFormatter = new Hashtable();
	private final static String STD_FORMAT = "yyyyMMdd-HHmmss" ;

	public static String calendarToString(Calendar c) {
		return calendarToString(c, STD_FORMAT);
	}
	
	public static String calendarToDayString(Calendar c) {
		return calendarToString(c, "yyyyMMdd");
	}

	private static String dateToString(Date d, String format) {
		if (d == null)
			d = new Date();
		DateFormat dateformatter = getDateFormat(format);
		return dateformatter.format(d);
	}

	public static String calendarToString(Calendar c, String format){
		Date d = new Date() ;
		d.setYear(c.get(Calendar.YEAR) - 1900) ;
		d.setMonth(c.get(Calendar.MONTH)) ;
		d.setDate(c.get(Calendar.DATE)) ;
		d.setHours(c.get(Calendar.HOUR_OF_DAY)) ;
		d.setMinutes(c.get(Calendar.MINUTE)) ;
		d.setSeconds(c.get(Calendar.SECOND)) ;
		return dateToString(d, format) ;
	}

	
	public static String currentSeoulToString() {
		return calendarToString(Calendar.getInstance(TimeZone.getTimeZone("Asia/Seoul")));
	}

	public static String currentDateToString(String format) {
		return dateToString(new Date(), format);
	}

	public static Date stringToDate(String s) {
		if (s.endsWith("+02"))
			s = s.substring(0, s.length() - 3);
		if (s.endsWith("+01"))
			s = s.substring(0, s.length() - 3);
		try {
			DateFormat dateformatter = getDateFormat(STD_FORMAT) ;
			if (s == null || s.length() < 1)
				return new Date();
			return dateformatter.parse(s);
		} catch (ParseException e) {
			throw new IllegalArgumentException(e);
		}
	}

    public static Calendar stringToCalendar( String s ){
        try {
			Calendar c = Calendar.getInstance();
			c.setTime( getDateFormat(STD_FORMAT).parse( s ) );
			return c;
		} catch (NullPointerException e) {
			throw new IllegalArgumentException(e);
		} catch (ParseException e) {
			throw new IllegalArgumentException(e);
		}
    }
    

    private static DateFormat getDateFormat(String format){
		DateFormat result = (DateFormat) dateFormatter.get(format);
		if (result == null) {
			result = new SimpleDateFormat(format);
			dateFormatter.put(format, result);
		}
    	return result ;
    }
    

	public static Date stringToDate(String s, String format) {
		if (s.endsWith("+02"))
			s = s.substring(0, s.length() - 3);
		if (s.endsWith("+01"))
			s = s.substring(0, s.length() - 3);
		try {
			DateFormat df = new SimpleDateFormat(format);
			// df.setTimeZone(TimeZone.getTimeZone("ECT"));
			// if (s == null || s.length() < 1) return new Date();
			return df.parse(s);
		} catch (Exception e) {
			throw new IllegalArgumentException(e);
		}
	}


	/*
	 * @deprecated use calendarToTime class method
	 */
	public Timestamp getTimestamp(Calendar cal) {
		if (cal == null)
			return null;
		long timeval = cal.getTime().getTime();
		return new Timestamp(timeval);
	}

	/*
	 * @deprecated use timeToCalendar class method
	 */
	public Calendar getCalendar(Timestamp t) {
		if (t == null)
			return null;
		Calendar cal = new GregorianCalendar();
		cal.setTimeInMillis(t.getTime());
		return cal;
	}

	public Calendar getCurrentCalendar() {
		Calendar now = new GregorianCalendar();
		return now;
	}

	// static methods ------------------------------
	public static Calendar timeToCalendar(Timestamp t) {
		if (t == null)
			return null;
		Calendar c = new GregorianCalendar();
		c.setTimeInMillis(t.getTime());
		return c;
	}

	public static Timestamp calendarToTime(Calendar c) {
		if (c == null)
			return null;
		// long timeval = c.getTime().getTime();
		return new Timestamp(c.getTimeInMillis());
	}

	public static Calendar dateToCalendar(java.util.Date d) {
		if (d == null)
			return null;
		Calendar c = new GregorianCalendar();
		c.setTime(d);
		return c;
	}

	public static java.util.Date calendarToDate(Calendar c) {
		if (c == null)
			return null;
		return new java.util.Date(c.getTimeInMillis());
	}

	public static Calendar longToCalendar(long misec) {
		Calendar c = new GregorianCalendar();
		c.setTimeInMillis(misec);
		return c;
	}

	public static long calendarToLong(Calendar c) {
		if (c == null)
			return 0;
		return c.getTimeInMillis();
	}

	public static String currentGMTToString() {
		Calendar c = new GregorianCalendar();
		return calendarToString(c) ;
	}

	public static String calendarToString(Calendar c, TimeZone tz) {
		c.setTimeZone(tz) ;
		return calendarToString(c) ;
	}

	public static String gmtStringToString(String utcDate, TimeZone tz) {
		return calendarToString(DateUtil.stringToCalendar(utcDate), tz) ;
	}

	public static String calendarToGMTString(Calendar c) {
		return calendarToString(c, TimeZone.getTimeZone("Etc/GMT"));
	}
	
	public static String calendarToSeoulString(Calendar c) {
		return calendarToString(c, TimeZone.getTimeZone("Asia/Seoul"));
	}

}
